@@ -27,7 +27,7 @@
          fixture_test/1, primitive_test/1, surefire_utf8_test/1,
          surefire_latin_test/1, surefire_c0_test/1, surefire_ensure_dir_test/1,
          stacktrace_at_timeout_test/1, scale_timeouts_test/1,
-         report_failed_setup_inparallel_test/1]).
+         report_failed_setup_inparallel_test/1, parse_commandline_test/1]).
 
 %% Two eunit tests:
 -export([times_out_test_/0, times_out_default_test/0]).
@@ -44,7 +44,8 @@ all() ->
     [app_test, appup_test, eunit_test, eunit_exact_test, primitive_test,
      fixture_test, surefire_utf8_test, surefire_latin_test, surefire_c0_test,
      surefire_ensure_dir_test, stacktrace_at_timeout_test,
-     scale_timeouts_test, report_failed_setup_inparallel_test].
+     scale_timeouts_test, report_failed_setup_inparallel_test,
+     parse_commandline_test].
 
 groups() ->
     [].
@@ -274,3 +275,51 @@ report_failed_setup_inparallel_test(_Config) ->
     eunit:test(Test,[verbose, {report, {eunit_test_listener, [self()]}}]),
     check_test_results(Test, #{skip => 0,cancel => 1,fail => 0,pass => 1}),
     ok.
+
+%% Eunit: Checks that eunit_data:parse_command_line correctly handles various command lines
+parse_commandline_test(_Config) ->
+    lists:foreach(
+        fun({Input, Expect}) ->
+            Output = eunit_data:parse_command_line(Input, []),
+            ?assertEqual(Expect, Output, lists:flatten(io_lib:format(
+                "Input=~0p expected=~0p output=~0p", [Input, Expect, Output])))
+        end,
+        [
+            %% Basic splitting and whitespace handling
+            {"", []},
+            {"ab", ["ab"]},
+            {"a", ["a"]},
+            {"a b   c", ["a", "b", "c"]},
+            {"  a  b c  ", ["a", "b", "c"]},
+            {"a\tb\nc", ["a", "b", "c"]},
+
+            %% Double-quoted sections (quotes removed)
+            {"a \"b c\" d", ["a", "b c", "d"]},
+            {"a \"b\tc\" d", ["a", "b\tc", "d"]},
+            {"a \"b\nc\" d", ["a", "b\nc", "d"]},
+            {"\"a b\" \"c d\"", ["a b", "c d"]},
+            {"\"\"", [""]}, % empty string in double quotes
+
+            %% Escapes inside double quotes
+            {"a \"b\\\"c\" d", ["a", "b\"c", "d"]},
+            {"a \"b\\\\c\" d", ["a", "b\\c", "d"]},
+
+            %% Single-quoted sections (quotes removed)
+            {"a 'b c' d", ["a", "b c", "d"]},
+            {"''", [""]},
+
+            %% Escapes inside single quotes (backslash escapes next char)
+            {"'it\\'s' ok", ["it's", "ok"]},
+            {"a 'b\\\\c' d", ["a", "b\\c", "d"]},
+
+            %% Unbalanced quotes: returned token keeps the dangling opening quote
+            {"a \"b c", ["a", "\"b c"]},
+            {"'b c", ["'b c"]},
+
+            %% Backslash outside quotes is literal + single quote test: parser
+            %% should return the following words separately
+            {"a\\ b", ["a\\", "b"]},
+            {"a ' b", ["a", "' b"]},
+            {"a ' b c", ["a", "' b c"]}
+        ]),
+    ok.
