@@ -1,7 +1,9 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2024. All Rights Reserved.
+%% SPDX-License-Identifier: Apache-2.0
+%%
+%% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -55,8 +57,8 @@ all() ->
      '0_RTT_handshake'].
 
 init_per_suite(Config) ->
-    catch crypto:stop(),
-    try (ok == crypto:start()) andalso ssl_test_lib:sufficient_crypto_support('tlsv1.3') of
+    catch application:stop(crypto),
+    try (ok == application:start(crypto)) andalso ssl_test_lib:sufficient_crypto_support('tlsv1.3') of
 	true ->
 	    ssl_test_lib:clean_start(),
             Config;
@@ -1435,11 +1437,11 @@ finished_verify_data(_Config) ->
 %% Internal functions ------------------------------------------------
 %%--------------------------------------------------------------------
 
-hexstr2int(S) ->
-    B = hexstr2bin(S),
-    Bits = byte_size(B) * 8,
-    <<Integer:Bits/integer>> = B,
-    Integer.
+%% hexstr2int(S) ->
+%%     B = hexstr2bin(S),
+%%     Bits = byte_size(B) * 8,
+%%     <<Integer:Bits/integer>> = B,
+%%     Integer.
 
 hexstr2bin(S) when is_binary(S) ->
     hexstr2bin(S, <<>>);
