@@ -1036,49 +1036,31 @@ multi_esc(Bin, Unicode) ->
     {_Cha, Tpl} = multi_hex_esc(Bin, Unicode),
     Tpl.
 
-multi_hex_esc(<<"x{",N,$},Rest/binary>>,Unicode) when ?is_hex_char(N) ->
-    Cha = trx(N),
-    case Unicode of
-	false ->
-	    {Cha, {<<Cha:8>>,Rest}};
-	_ ->
-	    {Cha, {int_to_utf8(Cha),Rest}}
+
+multi_hex_esc(<<"x{", Rest0/binary>>, Unicode) ->
+    case hex_num(Rest0, 6) of
+        {Cha, <<$}, Rest1/binary>>} ->
+            case {Unicode, Cha < 256} of
+                {false, true} ->
+                    {Cha, {<<Cha:8>>, Rest1}};
+                _ ->
+                    {Cha, {int_to_utf8(Cha), Rest1}}
+            end;
+        _Error ->
+            {no, no}
     end;
-multi_hex_esc(<<"x{",N,O,$},Rest/binary>>,Unicode) when (?is_hex_char(N) and
-                                                         ?is_hex_char(O)) ->
-    Cha = (trx(N) bsl 4) bor trx(O),
-    case Unicode of
-	false ->
-	    {Cha, {<<Cha:8>>,Rest}};
-	_ ->
-	    {Cha, {int_to_utf8(Cha),Rest}}
+multi_hex_esc(<<"N{U+", Rest0/binary>>, Unicode) ->
+    case hex_num(Rest0, 6) of
+        {Cha, <<$}, Rest1/binary>>} ->
+            case {Unicode, Cha < 256} of
+                {false, true} ->
+                    {Cha, {<<Cha:8>>, Rest1}};
+                _ ->
+                    {Cha, {int_to_utf8(Cha), Rest1}}
+            end;
+        _Error ->
+            {no, no}
     end;
-multi_hex_esc(<<"x{",N,O,P,$},Rest/binary>>,_) when (?is_hex_char(N) and
-                                                     ?is_hex_char(O) and
-                                                     ?is_hex_char(P)) ->
-    Cha = (trx(N) bsl 8) bor (trx(O) bsl 4) bor trx(P),
-    {Cha, {int_to_utf8(Cha),Rest}};
-multi_hex_esc(<<"x{",N,O,P,Q,$},Rest/binary>>,_) when (?is_hex_char(N) and
-                                                       ?is_hex_char(O) and
-                                                       ?is_hex_char(P) and
-                                                       ?is_hex_char(Q)) ->
-    Cha = (trx(N) bsl 12) bor (trx(O) bsl 8) bor (trx(P) bsl 4) bor trx(Q),
-    {Cha, {int_to_utf8(Cha),Rest}};
-multi_hex_esc(<<"x{",N,O,P,Q,R,$},Rest/binary>>,_) when (?is_hex_char(N) and
-                                                         ?is_hex_char(O) and
-                                                         ?is_hex_char(P) and
-                                                         ?is_hex_char(Q) and
-                                                         ?is_hex_char(R)) ->
-    Cha = (trx(N) bsl 16) bor (trx(O) bsl 12) bor (trx(P) bsl 8) bor (trx(Q) bsl 4) bor trx(R),
-    {Cha, {int_to_utf8(Cha),Rest}};
-multi_hex_esc(<<"x{",N,O,P,Q,R,S,$},Rest/binary>>,_) when (?is_hex_char(N) and
-                                                           ?is_hex_char(O) and
-                                                           ?is_hex_char(P) and
-                                                           ?is_hex_char(Q) and
-                                                           ?is_hex_char(R) and
-                                                           ?is_hex_char(S)) ->
-    Cha = (trx(N) bsl 20) bor (trx(O) bsl 16) bor (trx(P) bsl 12) bor (trx(Q) bsl 8) bor (trx(R) bsl 4) bor trx(S),
-    {Cha, {int_to_utf8(Cha),Rest}};
 multi_hex_esc(<<$x,N,O,Rest/binary>>,_) when (?is_hex_char(N) and
                                               ?is_hex_char(O)) ->
     Cha = (trx(N) bsl 4) bor trx(O),
@@ -1089,6 +1071,20 @@ multi_hex_esc(<<$x,N,Rest/binary>>,_) when ?is_hex_char(N) ->
 multi_hex_esc(_,_) ->
     {no, no}.
 
+hex_num(Bin, Maxlen) ->
+    hex_num(Bin, Maxlen, 0, 0).
+
+hex_num(Bin, MaxLen, MaxLen, Acc) ->
+    {Acc, Bin};
+hex_num(<<C, Rest/binary>>, Maxlen, Gotlen, Acc) when ?is_hex_char(C) ->
+    hex_num(Rest, Maxlen, Gotlen+1, (Acc bsl 4) bor trx(C));
+hex_num(Bin, _Maxlen, Gotlen, Acc) when Gotlen > 0 ->
+    {Acc, Bin};
+hex_num(Bin, _, 0, 0) ->
+    {error, "Expected hex number", Bin}.
+
+
+
 single_esc($") ->
     $";
 single_esc(?SPACE) ->
@@ -1433,19 +1429,45 @@ ranstring() ->
 re_compile(RE, Options) ->
     inc_counter(re_compile),
     put(re_compile_opts, Options),
-    re:compile(RE, Options).
+    case re:compile(RE, Options) of
+        {ok, Local} ->
+            {ok, Exported} = re:compile(RE, [export|Options]),
+            {ok, {Local, Exported}};
+        Error ->
+            Error = re:compile(RE, [export | Options])
+    end.
 
+
+re_run(Subj, {Local, Exported}, Opts) ->
+    Res = re_run(Subj, Local, Opts),
+    Imported1 = re:import(Exported),
+    Res = re_run(Subj, Imported1, Opts),
+    Imported2 = re:import(bump_exported_version(Exported)),
+    Res = re_run(Subj, Imported2, Opts);
 re_run(Subj, RE, Opts) ->
     %%io:format("re:run(~p, ~p, ~p)\n", [Subj, RE, Opts]),
     inc_counter(re_run),
     put(re_run_opts, Opts),
     re:run(Subj, RE, Opts).
 
+re_replace(Subj, {Local, Exported}, Repl, Opts) ->
+    Res = re_replace(Subj, Local, Repl, Opts),
+    Imported = re:import(Exported),
+    Res = re_replace(Subj, Imported, Repl, Opts);
 re_replace(Subj, RE, Repl, Opts) ->
     inc_counter(re_replace),
     put(re_run_opts, Opts),
     re:replace(Subj, RE, Repl, Opts).
 
+bump_exported_version(Exported1) ->
+    {re_exported_pattern, Hdr1, RE, Opts, Enc1} = Exported1,
+    <<Magic:32, Maj:16, Min:16/little, EncRest/binary>> = Enc1,
+    Enc2 = <<Magic:32, Maj:16, (Min+1):16/little, EncRest/binary>>,
+    <<"re-PCRE2", _CRC1:32, HdrRest/binary>> = Hdr1,
+    CRC2 = erlang:crc32(Enc2),
+    Hdr2 = <<"re-PCRE2", CRC2:32, HdrRest/binary>>,
+    {re_exported_pattern, Hdr2, RE, Opts, Enc2}.
+
 used_options() ->
     RunOpts = get(re_run_opts),
     case get(re_compile_opts) of
